import logging
from os.path import join
from django.template.loader import render_to_string

from .toontjehoger_1_mozart import toontjehoger_ranks
from experiment.actions import Plink, Explainer, Step, Score, Final, StartSession, Playlist, Info
from experiment.actions.form import RadiosQuestion
from .base import Base

from experiment.actions.utils import combine_actions
from experiment.utils import non_breaking_spaces

from result.utils import prepare_result

logger = logging.getLogger(__name__)


class ToontjeHoger3Plink(Base):
    ID = 'TOONTJE_HOGER_3_PLINK'
    TITLE = ""
    SCORE_MAIN_CORRECT = 10
    SCORE_MAIN_WRONG = 0
    SCORE_EXTRA_1_CORRECT = 4
    SCORE_EXTRA_2_CORRECT = 4
    SCORE_EXTRA_WRONG = 0
    
    def first_round(self, experiment):
        """Create data for the first experiment rounds."""

        # 1. Explain game.
        explainer = Explainer(
            instruction="Muziekherkenning",
            steps=[
                Step("Je krijgt {} zeer korte muziekfragmenten te horen.".format(
                    experiment.rounds)),
                Step("Ken je het nummer? Noem de juiste artiest en titel!"),
                Step(
                    "Weet je het niet? Beantwoord dan extra vragen over de tijdsperiode en emotie van het nummer.")
            ],
            step_numbers=True,
            button_label="Start"
        )

        # 2. Choose playlist.
        playlist = Playlist(experiment.playlists.all())

        # 3. Start session.
        start_session = StartSession()

        return [
            explainer,
            playlist,
            start_session
        ]
    
    def next_round(self, session, request_session=None):
        """Get action data for the next round"""

        rounds_passed = session.rounds_passed()

        # Round 1
        if rounds_passed == 0:
            # No combine_actions because of inconsistent next_round array wrapping in first round
            return self.get_plink_round(session)

        # Round 2-experiments.rounds
        if rounds_passed < session.experiment.rounds:
            return [*self.get_score(session), *self.get_plink_round(session)]

        # Final
        return self.get_final_round(session)
  
    def get_score_message(self, session):
        last_result = session.last_result()

        if not last_result:
            logger.error("No last result")
            return ""

        data = last_result.load_json_data()

        # Section
        section = last_result.section
        if not section:
            logger.error("Result without section")
            return ""

        # Option 1. Main question
        main_question = Plink.extract_main_question(data)

        if main_question:
            if main_question == last_result.expected_response:
                return "Goedzo! Je hoorde inderdaad {} van {}.".format(non_breaking_spaces(section.song.name), non_breaking_spaces(section.song.artist))

            return "Helaas! Je hoorde {} van {}.".format(non_breaking_spaces(section.song.name), non_breaking_spaces(section.song.artist))

        # Option 2. Extra questions
        extra_questions = Plink.extract_extra_questions(data)

        # No extra questions? Return just an empty string
        if not extra_questions:
            return ""

        # Feedback prefix

        # - All points
        feedback_prefix = "Goedzo!"

        # - Partial score or all questions wrong
        all_wrong_score = last_result.score == 2 * self.SCORE_EXTRA_WRONG
        only_half_score = last_result.score < self.SCORE_EXTRA_1_CORRECT + \
            self.SCORE_EXTRA_2_CORRECT if not all_wrong_score else False

        if all_wrong_score:
            feedback_prefix = "Helaas!"
        elif only_half_score:
            feedback_prefix = "Deels goed!"

        # Get section info
        section_details = section.group.split(";")
        time_period = section_details[0] if len(
            section_details) >= 1 else "?"
        time_period = time_period.replace("s", "'s")
        emotion = section_details[1] if len(section_details) >= 2 else "?"

        # Construct final feedback message
        question_part = "Het nummer komt uit de {} en de emotie is {}.".format(
            time_period, emotion)
        section_part = "Je hoorde {} van {}.".format(
            non_breaking_spaces(section.song.name), non_breaking_spaces(section.song.artist))

        # The \n results in a linebreak
        feedback = "{} {} \n {}".format(
            feedback_prefix, question_part, section_part)
        return feedback
   
    def get_score(self, session):
        feedback = self.get_score_message(session)

        # Return score view
        config = {'show_total_score': True}
        score = Score(session, config=config, feedback=feedback)

        return [score]
    
    def get_plink_round(self, session):

        # Config
        # -----------------

        # Get all song sections
        all_sections = session.all_sections()
        choices = {}
        for section in all_sections:
            label = section.song_label()
            choices[section.pk] = label

        # Get section to recognize
        section = session.section_from_unused_song()
        if section == None:
            raise Exception("Error: could not find section")

        expected_response = section.pk

        # Extra questions intro
        # --------------------
        extra_questions_intro = Explainer(
            instruction="Tussenronde",
            steps=[
                Step("Jammer dat je de artiest en titel van dit nummer niet weet!"),
                Step(
                    "Verdien extra punten door twee extra vragen over het nummer te beantwoorden."),
            ],
            button_label="Start"
        )

        # Plink round
        # --------------------
        extra_questions = [self.get_optional_question1(
            session), self.get_optional_question2(session)]

        plink = Plink(
            section=section,
            title=self.TITLE,
            result_id=prepare_result(
                'plink', session, section=section, expected_response=expected_response
            ),
            main_question="Noem de artiest en de titel van het nummer",
            choices=choices,
            submit_label="Volgende",
            dont_know_label="Ik weet het niet",
            extra_questions=extra_questions,
            extra_questions_intro=extra_questions_intro
        )

        return [plink]
 
    def get_optional_question1(self, session):

        # Config
        # -----------------

        # Question
        periods = ["60's", "70's", "80's", "90's", "00's", "10's", "20's"]
        period_choices = {}
        for period in periods:
            period_choices[period.replace("'", "")] = period

        question = RadiosQuestion(
            question="Wanneer is het nummer uitgebracht?",
            key='time_period',
            choices=period_choices,
            submits=False
        )

        return question
 
    def get_optional_question2(self, session):

        # Question
        emotions = ['vrolijk', 'droevig', 'boosheid', 'angst', 'tederheid']
        emotion_choices = {}
        for emotion in emotions:
            emotion_choices[emotion] = emotion.capitalize()

        question = RadiosQuestion(
            question="Welke emotie past bij dit nummer?",
            key='emotion',
            choices=emotion_choices,
            submits=True
        )

        return question
    
    def calculate_score(self, result, data):
        """
        Calculate score, based on the data field

        e.g. only main question answered
        {
            main_question: "100",
            extra_questions: []
        }

        e.g. only main question answered
        {
            main_question: "",
            extra_questions: ["60s","vrolijk"]
        }

        """
        main_question = Plink.extract_main_question(data)

        # Participant guessed the artist/title:
        if main_question != "":
            result.given_response = main_question
            result.save()
            return self.SCORE_MAIN_CORRECT if result.expected_response == result.given_response else self.SCORE_MAIN_WRONG

        # Handle extra questions data
        extra_questions = Plink.extract_extra_questions(data)
        if extra_questions:
            section = result.section
            if section is None:
                logger.error("Error: No section on result")
                return 0

            score = 0

            # Check if the given answers
            # e.g section.group = 60s;vrolijk (time_period;emotion)
            for index, answer in enumerate(extra_questions):
                points_correct = self.SCORE_EXTRA_1_CORRECT if index == 0 else self.SCORE_EXTRA_2_CORRECT
                score += points_correct if answer and (
                    answer in section.group) else self.SCORE_EXTRA_WRONG

            return score

        # Should not happen
        logger.error("Error: could not calculate score")
        return 0
 
    def get_final_round(self, session):

        # Finish session.
        session.finish()
        session.save()

        # Score
        score = self.get_score(session)

        # Final
        final_text = "Goed gedaan, jouw muziekherkenning is uitstekend!" if session.final_score >= 4 * \
            self.SCORE_MAIN_CORRECT else "Dat bleek toch even lastig!"
        final = Final(
            session=session,
            final_text=final_text,
            rank=toontjehoger_ranks(session),
            button={'text': 'Wat hebben we getest?'}
        )

        # Info page
        body = render_to_string(
            join('info', 'toontjehoger', 'experiment3.html'))
        info = Info(
            body=body,
            heading="Muziekherkenning",
            button_label="Terug naar ToontjeHoger",
            button_link="/toontjehoger"
        )

        return [*score, final, info]
